<?php

namespace Drupal\graphql_config\Plugin\GraphQL\Fields;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\graphql\GraphQL\Execution\ResolveContext;
use Drupal\graphql\Plugin\GraphQL\Fields\FieldPluginBase;
use GraphQL\Type\Definition\ResolveInfo;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Returns a config item's value.
 *
 * Scalars are returned as-is
 *
 * @GraphQLField(
 *   id = "config_item",
 *   secure = true,
 *   name = "configItem",
 *   arguments = {
 *     "namespace" = "String!",
 *     "key" = "String!",
 *   },
 *   description = @Translation("A simple config item. Scalar values are returned as-is. Other data types are json-encoded."),
 *   type = "String"
 * )
 */
class ConfigItem extends FieldPluginBase implements ContainerFactoryPluginInterface {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * ConfigItem constructor.
   *
   * @param array $configuration
   * @param $plugin_id
   * @param $plugin_definition
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConfigFactoryInterface $configFactory, AccountProxyInterface $currentUser) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configFactory = $configFactory;
    $this->currentUser = $currentUser;
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
      $container->get('current_user')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function resolveValues($value, array $args, ResolveContext $context, ResolveInfo $info) {
    if ($this->currentUser->hasPermission('read config items via graphl')) {
      $config = $this->configFactory->get($args['namespace']);
      if ($config) {
        $value = $config->get($args['key']);
        if ($value) {
          yield is_scalar($value) ? $value : json_encode($value);
        }
      }
    }
  }

}
